% Blind inversion of Wiener systems through mutual information mininization - CLONALG
% Date: 11/01/2016


close all
clear all
clc

tic;
%%CLONALG parameters
niter         = 300;       % number of CLONALG iterations
alpha         = .1;
beta          = .1;
rho           = 4;
Nind          = 50;


%% Parameters definition
N             = 2000;        % Number of samples
N_coeff_poly  = 5;   % Compensating polynomial's size
M             = 3;    %FIR part
Q             = 3;    %IIR part - se quiser desabilitar - Q=0
size_eq       = [M Q N_coeff_poly];% Equalizer's length


modo = 'falante'; %falante - prints iteration info


h = [1 .6];                  % channel
h = h/norm(h);

tapmax = 20;

  
RandStream.setGlobalStream(RandStream('mt19937ar','seed',3));
%% Convolution 

s = rand(1,N); 
s = s - mean(s);
s = s./std(s);  % Sources

e = filter(h,1,s);
%x = (abs(e).^(1/3)).*sign(e);
x = tanh(3*e) + 0.1*e; %


%cost function
func = @(Ab)custo_mirate(Ab,x);



%% CLONALG algorithm    
[solution,Ab,f] = clonalg_wh(func,alpha,beta,rho,Nind,size_eq,niter,modo);

u = polyval(kron(solution.g,[1 0]),x);
u = u - mean(u);
u = u./std(u); %normalizacao e centralizacao

y = filter(solution.FIR,[1 solution.IIR],u);
py = std(y);

y = y./py; %normalizacao   
solution.FIR = (solution.FIR)./py;

[mse,snr,btap] = desempenho(s,y,tapmax);


figure;
subplot(1,3,1);plot(e,x,'.');
x_handle = xlabel('e(n)');y_handle = ylabel('x(n)');t_handle = title('Nonlinear distortion f(\cdot)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
set(x_handle,'FontSize',16,'FontWeight','bold')
set(y_handle,'FontSize',16,'FontWeight','bold')
subplot(1,3,2);plot(x,u,'.');x_handle = xlabel('x(n)');y_handle = ylabel('u(n)');t_handle = title('Nonlinear correction g(\cdot)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
set(x_handle,'FontSize',16,'FontWeight','bold')
set(y_handle,'FontSize',16,'FontWeight','bold')
subplot(1,3,3);plot(e,u,'.');x_handle = xlabel('e(n)');y_handle = ylabel('u(n)');t_handle = title('g\circ f');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
set(x_handle,'FontSize',16,'FontWeight','bold')
set(y_handle,'FontSize',16,'FontWeight','bold')

figure;
subplot(1,2,1);hist(s,ceil(length(s)^(1/3)));t_handle = title('Histogram of s(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(1,2,2);hist(y,ceil(length(y)^(1/3)));t_handle = title('Histogram of y(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')

figure;
subplot(3,1,1);plot(1:N,s);t_handle = title('s(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(3,1,2);plot(1:N,-y);t_handle = title('-y(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(3,1,3);plot(1:N-btap,s(1:end-btap)+y(btap+1:end));
t_handle = title('s(n)+y(n)');axis([0 N-btap -2 2]);
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')

figure;
subplot(3,1,1);plot(1:N,s);t_handle = title('s(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(3,1,2);plot(1:N,y);t_handle = title('y(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(3,1,3);plot(1:N-btap,s(1:end-btap)-y(btap+1:end));t_handle = title('s(n)-y(n)');axis([0 N -2 2]);
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')


figure;
subplot(1,3,1);impz(h);t_handle = title('h(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(1,3,2);impz(solution.FIR,[1 solution.IIR]);t_handle = title('w(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')
subplot(1,3,3);impz(conv(solution.FIR,h),[1 solution.IIR]);t_handle = title('h(n)\ast w(n)');
set(gca,'FontSize',16);
set(t_handle,'FontSize',16,'FontWeight','bold')

figure;
plot(1:N-btap,s(1:end-btap),1:N-btap,y(btap+1:end));l_handle = legend('s(n)','y(n)');
set(gca,'FontSize',16);
set(l_handle,'FontSize',16,'FontWeight','bold')

fprintf('------Metrics------\nP = %f, SNR = %f, Delay = %d\n',10*log10(mse),snr,btap);

toc












