
%funcao que calcula a taxa de IM da saida do sistema de Hammerstein
%g: coeficientes do polinomio de expoentes impares
%w1 e w2: numerador e denominador do filtro linear
%x: sequencia de entrada no sistema de Hammerstein
function [MI,y] = mirate(w1,w2,g,x)

N = 1024; %resolucao da FFT
inicio = 1;

z = polyval(g,x(inicio:end)); %z: sinal de saida da nao-linearidade g(.)
pz = std(z);
g = g./pz;

z = z - mean(z); %centralizacao
z = z./pz; %normalizacao

y = filter(w1,w2,z);   %y: sinal de saida do sistema de Hammerstein                     
if(max(abs(y))>100 || sum(y)==0) %filtro divergiu
    MI = NaN;
else
    py = std(y);
    y = y./py; %normalizacao
    
    glinha = polyder(g);
    glinhax = polyval(glinha,x(inicio:end));

    Elogg = mean(log(glinhax));
    
    H = oe_entropy(y);

    sinal = impz(w1./py,w2);            
    W = abs(fft(sinal,N));

    Int = sum(log(W))/N;
        
    MI = H - Int - Elogg;                       
end


end

            